/* -*- compile-command: "cd ../../../../../; ant install"; -*- */
/*
 * Copyright 2009-2010 by Eric House (xwords@eehouse.org).  All
 * rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.eehouse.android.xw4;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.app.ExpandableListActivity;
import android.database.DataSetObserver;
import android.os.Bundle;
import android.widget.Button;
import android.widget.TextView;
import android.widget.AdapterView;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.res.Resources;
import android.content.SharedPreferences;
import android.view.View;
import android.view.ViewGroup;
import android.view.ContextMenu;
import android.view.MenuItem;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.MenuInflater;
import android.widget.ExpandableListAdapter;
import android.widget.ExpandableListView;
import android.widget.ExpandableListView.ExpandableListContextMenuInfo;
import android.widget.Toast;
import android.preference.PreferenceManager;
import android.net.Uri;
import junit.framework.Assert;

import org.eehouse.android.xw4.jni.XwJNI;
import org.eehouse.android.xw4.jni.JNIUtilsImpl;
import org.eehouse.android.xw4.jni.CommonPrefs;

public class DictsActivity extends ExpandableListActivity 
    implements View.OnClickListener, XWListItem.DeleteCallback,
               SDCardWatcher.SDCardNotifiee, DlgDelegate.DlgClickNotify {

    private static final String DICT_DOLAUNCH = "do_launch";
    private static final String DICT_LANG_EXTRA = "use_lang";
    private static final String DICT_NAME_EXTRA = "use_dict";
    private static final String PACKED_POSITION = "packed_position";
    private static final String DELETE_DICT = "delete_dict";
    private static final String NAME = "name";
    private static final String LANG = "lang";
    private static final String MOVEFROMLOC = "movefromloc";
    private static final String MOVETOLOC = "movetoloc";

    // For new callback alternative
    private static final int DELETE_DICT_ACTION = 1;

    private static final int PICK_STORAGE = DlgDelegate.DIALOG_LAST + 1;
    private static final int MOVE_DICT = DlgDelegate.DIALOG_LAST + 2;
    private static final int SET_DEFAULT = DlgDelegate.DIALOG_LAST + 3;
    private int m_lang = 0;
    private String[] m_langs;
    private String m_name = null;
    private String m_deleteDict = null;
    private String m_download;
    private ExpandableListView m_expView;
    private DlgDelegate m_delegate;
    private String[] m_locNames;
    private DictListAdapter m_adapter;

    private long m_packedPosition;
    private GameUtils.DictLoc m_moveFromLoc;
    private GameUtils.DictLoc m_moveToLoc;
    private SDCardWatcher m_cardWatcher;

    private LayoutInflater m_factory;

    private class DictListAdapter implements ExpandableListAdapter {
        private Context m_context;
        private XWListItem[][] m_cache;

        public DictListAdapter( Context context ) {
            //super( context, m_dicts.length );
            m_context = context;
        }

        public boolean areAllItemsEnabled() { return false; }

        public Object getChild(int groupPosition, int childPosition)
        {
            return null;
        }

        public long getChildId( int groupPosition, int childPosition )
        {
            return childPosition;
        }

        public View getChildView( int groupPosition, int childPosition, 
                                  boolean isLastChild, View convertView, 
                                  ViewGroup parent)
        {
            return getChildView( groupPosition, childPosition );
        }

        private View getChildView( int groupPosition, int childPosition )
        {
            XWListItem view = null;
            if ( null != m_cache && null != m_cache[groupPosition] ) {
                view = m_cache[groupPosition][childPosition];
            }

            if ( null == view ) {
                int lang = (int)getGroupId( groupPosition );
                String[] dicts = DictLangCache.getHaveLang( m_context, lang );
                String text;
                boolean canDelete = false;
                if ( null != dicts && childPosition < dicts.length ) {
                    text = dicts[childPosition];
                    canDelete = !GameUtils.dictIsBuiltin( DictsActivity.this,
                                                          text );
                } else {
                    text = m_download;
                }
                view = (XWListItem)m_factory.inflate( R.layout.list_item, null );
                view.setText( text );
                if ( canDelete ) {
                    view.setDeleteCallback( DictsActivity.this );
                }

                GameUtils.DictLoc loc = 
                    GameUtils.getDictLoc( DictsActivity.this, text );
                view.setComment( m_locNames[loc.ordinal()] );
                view.cache( loc );

                addToCache( groupPosition, childPosition, view );
            }
            return view;
        }

        public int getChildrenCount( int groupPosition )
        {
            int lang = (int)getGroupId( groupPosition );
            String[] dicts = DictLangCache.getHaveLang( m_context, lang );
            int result = 0; // 1;     // 1 for the download option
            if ( null != dicts ) {
                result += dicts.length;
            }
            return result;
        }

        public long getCombinedChildId( long groupId, long childId )
        {
            return groupId << 16 | childId;
        }

        public long getCombinedGroupId( long groupId )
        {
            return groupId;
        }

        public Object getGroup( int groupPosition )
        {
            return null;
        }

        public int getGroupCount()
        {
            return m_langs.length;
        }

        public long getGroupId( int groupPosition )
        {
            int lang = DictLangCache.getLangLangCode( m_context, 
                                                      m_langs[groupPosition] );
            return lang;
        }

        public View getGroupView( int groupPosition, boolean isExpanded, 
                                  View convertView, ViewGroup parent )
        {
            View row = 
                Utils.inflate(DictsActivity.this, 
                              android.R.layout.simple_expandable_list_item_1 );
            TextView view = (TextView)row.findViewById( android.R.id.text1 );
            view.setText( m_langs[groupPosition] );
            return view;
        }

        public boolean hasStableIds() { return false; }
        public boolean isChildSelectable( int groupPosition, 
                                          int childPosition ) { return true; }
        public boolean isEmpty() { return false; }
        public void onGroupCollapsed(int groupPosition){}
        public void onGroupExpanded(int groupPosition){}
        public void registerDataSetObserver( DataSetObserver obs ){}
        public void unregisterDataSetObserver( DataSetObserver obs ){}

        protected XWListItem getSelChildView()
        {
            int groupPosition = 
                ExpandableListView.getPackedPositionGroup( m_packedPosition );
            int childPosition = 
                ExpandableListView.getPackedPositionChild( m_packedPosition );
            return (XWListItem)getChildView( groupPosition, childPosition );
        }

        private void addToCache( int group, int child, XWListItem view )
        {
            if ( null == m_cache ) {
                m_cache = new XWListItem[getGroupCount()][];
            }
            if ( null == m_cache[group] ) {
                m_cache[group] = new XWListItem[getChildrenCount(group)];
            }
            Assert.assertTrue( null == m_cache[group][child] );
            m_cache[group][child] = view;
        }
    }

    @Override
    protected Dialog onCreateDialog( int id )
    {
        DialogInterface.OnClickListener lstnr;
        Dialog dialog;
        String format;
        String message;
        boolean doRemove = true;

        switch( id ) {
        case PICK_STORAGE:
            lstnr = new DialogInterface.OnClickListener() {
                    public void onClick( DialogInterface dlg, int item ) {
                        startDownload( m_lang, m_name, item != 
                                       DialogInterface.BUTTON_POSITIVE );
                    }
                };

            dialog = new AlertDialog.Builder( this )
                .setTitle( R.string.storeWhereTitle )
                .setMessage( R.string.storeWhereMsg )
                .setPositiveButton( R.string.button_internal, lstnr )
                .setNegativeButton( R.string.button_sd, lstnr )
                .create();
            break;
        case MOVE_DICT:
            lstnr = new DialogInterface.OnClickListener() {
                    public void onClick( DialogInterface dlg, int item ) {
                        XWListItem rowView = m_adapter.getSelChildView();
                        if ( GameUtils.moveDict( DictsActivity.this,
                                                 rowView.getText(),
                                                 m_moveFromLoc,
                                                 m_moveToLoc ) ) {
                            rowView.
                                setComment( m_locNames[m_moveToLoc.ordinal()]);
                            rowView.cache( m_moveToLoc );
                            rowView.invalidate();
                        } else {
                            Utils.logf( "moveDict failed" );
                        }
                    }
                };
            format = getString( R.string.move_dictf );
            message = String.format( format, 
                                     m_adapter.getSelChildView().getText(),
                                     m_locNames[ m_moveFromLoc.ordinal() ],
                                     m_locNames[ m_moveToLoc.ordinal() ] );

            dialog = new AlertDialog.Builder( this )
                .setMessage( message )
                .setPositiveButton( R.string.button_ok, lstnr )
                .setNegativeButton( R.string.button_cancel, null )
                .create();
            break;
        case SET_DEFAULT:
            lstnr = new DialogInterface.OnClickListener() {
                    public void onClick( DialogInterface dlg, int item ) {
                        if ( DialogInterface.BUTTON_NEGATIVE == item
                             || DialogInterface.BUTTON_POSITIVE == item ) {
                            setDefault( R.string.key_default_dict );
                        }
                        if ( DialogInterface.BUTTON_NEGATIVE == item 
                             || DialogInterface.BUTTON_NEUTRAL == item ) {
                            setDefault( R.string.key_default_robodict );
                        }
                    }
                };
            XWListItem rowView = m_adapter.getSelChildView();
            String lang = 
                DictLangCache.getLangName( this, rowView.getText() );
            format = getString( R.string.set_default_messagef );
            message = String.format( format, lang );
            dialog = new AlertDialog.Builder( this )
                .setTitle( R.string.query_title )
                .setMessage( message )
                .setPositiveButton( R.string.button_default_human, lstnr )
                .setNeutralButton( R.string.button_default_robot, lstnr )
                .setNegativeButton( R.string.button_default_both, lstnr )
                .create();
            break;
        default:
            dialog = m_delegate.onCreateDialog( id );
            doRemove = false;
            break;
        }

        if ( doRemove && null != dialog ) {
            Utils.setRemoveOnDismiss( this, dialog, id );
        }

        return dialog;
    } // onCreateDialog

    @Override
    protected void onPrepareDialog( int id, Dialog dialog )
    {
        super.onPrepareDialog( id, dialog );
        m_delegate.onPrepareDialog( id, dialog );
    }

    @Override
    protected void onCreate( Bundle savedInstanceState ) 
    {
        super.onCreate( savedInstanceState );
        getBundledData( savedInstanceState );

        Resources res = getResources();
        m_locNames = res.getStringArray( R.array.loc_names );

        m_delegate = new DlgDelegate( this, this, savedInstanceState );
        m_factory = LayoutInflater.from( this );

        m_download = getString( R.string.download_dicts );
            
        setContentView( R.layout.dict_browse );
        m_expView = getExpandableListView();
        registerForContextMenu( m_expView );

        Button download = (Button)findViewById( R.id.download );
        download.setOnClickListener( this );

        mkListAdapter();

        // showNotAgainDlg( R.string.not_again_dicts, 
        //                  R.string.key_notagain_dicts );

        Intent intent = getIntent();
        if ( null != intent ) {
            boolean downloadNow = intent.getBooleanExtra( DICT_DOLAUNCH, false );
            if ( downloadNow ) {
                int lang = intent.getIntExtra( DICT_LANG_EXTRA, 0 );
                String name = intent.getStringExtra( DICT_NAME_EXTRA );
                askStartDownload( lang, name );
            }
        }
    }

    @Override
    protected void onResume()
    {
        super.onResume();
        m_cardWatcher = new SDCardWatcher( this, this );
        mkListAdapter();
        expandGroups();
    }

    @Override
    protected void onSaveInstanceState( Bundle outState ) 
    {
        super.onSaveInstanceState( outState );
        m_delegate.onSaveInstanceState( outState );

        outState.putLong( PACKED_POSITION, m_packedPosition );
        outState.putString( NAME, m_name );
        outState.putInt( LANG, m_lang );
        outState.putString( DELETE_DICT, m_deleteDict );
        if ( null != m_moveFromLoc ) {
            outState.putInt( MOVEFROMLOC, m_moveFromLoc.ordinal() );
        }
        if ( null != m_moveToLoc ) {
            outState.putInt( MOVETOLOC, m_moveToLoc.ordinal() );
        }
    }

    private void getBundledData( Bundle savedInstanceState )
    {
        if ( null != savedInstanceState ) {
            m_packedPosition = savedInstanceState.getLong( PACKED_POSITION );
            m_name = savedInstanceState.getString( NAME );
            m_lang = savedInstanceState.getInt( LANG );
            m_deleteDict = savedInstanceState.getString( DELETE_DICT );

            int tmp = savedInstanceState.getInt( MOVEFROMLOC, -1 );
            if ( -1 != tmp ) {
                m_moveFromLoc = GameUtils.DictLoc.values()[tmp];
            }
            tmp = savedInstanceState.getInt( MOVETOLOC, -1 );
            if ( -1 != tmp ) {
                m_moveToLoc = GameUtils.DictLoc.values()[tmp];
            }
        }
    }

    @Override
    protected void onPause() {
        m_cardWatcher.close();
        m_cardWatcher = null;
        super.onPause();
    }

    public void onClick( View v ) 
    {
        askStartDownload( 0, null );
    }

    @Override
    public void onCreateContextMenu( ContextMenu menu, View view, 
                                     ContextMenuInfo menuInfo ) 
    {
        super.onCreateContextMenu( menu, view, menuInfo );

        ExpandableListView.ExpandableListContextMenuInfo info
            = (ExpandableListView.ExpandableListContextMenuInfo)menuInfo;
        long packedPosition = info.packedPosition;
        int childPosition = ExpandableListView.
            getPackedPositionChild( packedPosition );
        // int groupPosition = ExpandableListView.
        //     getPackedPositionGroup( packedPosition );
        // Utils.logf( "onCreateContextMenu: group: %d; child: %d", 
        //             groupPosition, childPosition );

        // We don't have a menu yet for languages, just for their dict
        // children
        if ( childPosition >= 0 ) {
            MenuInflater inflater = getMenuInflater();
            inflater.inflate( R.menu.dicts_item_menu, menu );
            
            XWListItem row = (XWListItem)info.targetView;
            GameUtils.DictLoc loc = (GameUtils.DictLoc)row.getCached();
            if ( loc == GameUtils.DictLoc.BUILT_IN
                 || ! GameUtils.haveWriteableSD() ) {
                menu.removeItem( R.id.dicts_item_move );
            }

            String fmt = getString(R.string.game_item_menu_titlef );
            menu.setHeaderTitle( String.format( fmt, row.getText() ) );
        }
    }
   
    @Override
    public boolean onContextItemSelected( MenuItem item ) 
    {
        boolean handled = false;
        ExpandableListContextMenuInfo info = null;
        try {
            info = (ExpandableListContextMenuInfo)item.getMenuInfo();
        } catch (ClassCastException e) {
            Utils.logf( "bad menuInfo: %s", e.toString() );
            return false;
        }
        
        m_packedPosition = info.packedPosition;

        int id = item.getItemId();
        switch( id ) {
        case R.id.dicts_item_move:
            askMoveDict( (XWListItem)info.targetView );
            break;
        case R.id.dicts_item_select:
            showDialog( SET_DEFAULT );
            break;
        case R.id.dicts_item_details:
            Utils.notImpl( this );
            break;
        }

        return handled;
    }

    private void setDefault( int keyId )
    {
        XWListItem view = m_adapter.getSelChildView();
        SharedPreferences sp
            = PreferenceManager.getDefaultSharedPreferences( this );
        SharedPreferences.Editor editor = sp.edit();
        String key = getString( keyId );
        String name = view.getText();
        editor.putString( key, name );
        editor.commit();
    }

    // Move dict.  Put up dialog asking user to confirm move from XX
    // to YY.  So we need both XX and YY.  There may be several
    // options for YY?
    private void askMoveDict( XWListItem item )
    {
        m_moveFromLoc = (GameUtils.DictLoc)item.getCached();
        if ( m_moveFromLoc == GameUtils.DictLoc.INTERNAL ) {
            m_moveToLoc = GameUtils.DictLoc.EXTERNAL;
        } else {
            m_moveToLoc = GameUtils.DictLoc.INTERNAL;
        }

        showDialog( MOVE_DICT );
    }

    // XWListItem.DeleteCallback interface
    public void deleteCalled( int myPosition, String dict )
    {
        int code = DictLangCache.getDictLangCode( this, dict );
        String lang = DictLangCache.getLangName( this, code );
        int nGames = DBUtils.countGamesUsing( this, code );
        String msg = String.format( getString( R.string.confirm_delete_dictf ),
                                    dict );
        m_deleteDict = dict;

        if ( nGames > 0 ) {
            int fmt;
            if ( 1 == DictLangCache.getHaveLang( this, code ).length ) {
                fmt = R.string.confirm_deleteonly_dictf;
            } else {
                fmt = R.string.confirm_deletemore_dictf;
            }
            msg += String.format( getString(fmt), lang );
        }

        m_delegate.showConfirmThen( msg, DELETE_DICT_ACTION );
    }

    // SDCardWatcher.SDCardNotifiee interface
    public void cardMounted()
    {
        mkListAdapter();
        expandGroups();
    }

    // DlgDelegate.DlgClickNotify interface
    public void dlgButtonClicked( int id, int which )
    {
        switch( id ) {
        case DELETE_DICT_ACTION:
            if ( DialogInterface.BUTTON_POSITIVE == which ) {
                deleteDict( m_deleteDict );
            }
            break;
        default:
            Assert.fail();
        }
    }

    private void deleteDict( String dict )
    {
        GameUtils.deleteDict( this, dict );
        DictLangCache.inval( this, dict, false );
        mkListAdapter();
    }

    private void askStartDownload( int lang, String name )
    {
        if ( GameUtils.haveWriteableSD() ) {
            m_lang = lang;
            m_name = name;
            showDialog( PICK_STORAGE );
        } else {
            startDownload( lang, name, false );
        }
    }

    private void startDownload( int lang, String name, boolean toSD )
    {
        DictImportActivity.setUseSD( toSD );
        try {
            startActivity( mkDownloadIntent( this, lang, name ) );
        } catch ( android.content.ActivityNotFoundException anfe ) {
            Toast.makeText( this, R.string.no_download_warning, 
                            Toast.LENGTH_SHORT).show();
        }
    }

    private void mkListAdapter()
    {
        m_langs = DictLangCache.listLangs( this );
        m_adapter = new DictListAdapter( this );
        setListAdapter( m_adapter );
    }

    private void expandGroups()
    {
        for ( int ii = 0; ii < m_langs.length; ++ii ) {
            m_expView.expandGroup( ii );
        }
    }

    private static Intent mkDownloadIntent( Context context,
                                            int lang, String dict )
    {
        String dict_url = CommonPrefs.getDefaultDictURL( context );
        if ( 0 != lang ) {
            dict_url += "/" + DictLangCache.getLangName( context, lang );
        }
        if ( null != dict ) {
            dict_url += "/" + dict + XWConstants.DICT_EXTN;
        }
        Uri uri = Uri.parse( dict_url );
        Intent intent = new Intent( Intent.ACTION_VIEW, uri );
        intent.setFlags( Intent.FLAG_ACTIVITY_NEW_TASK );
        return intent;
    }

    public static void launchAndDownload( Activity activity, int lang, 
                                          String name )
    {
        Intent intent = new Intent( activity, DictsActivity.class );
        intent.putExtra( DICT_DOLAUNCH, true );
        if ( lang > 0 ) {
            intent.putExtra( DICT_LANG_EXTRA, lang );
        }
        if ( null != name ) {
            Assert.assertTrue( lang != 0 );
            intent.putExtra( DICT_NAME_EXTRA, name );
        }

        activity.startActivity( intent );
    }

    public static void launchAndDownload( Activity activity, int lang )
    {
        launchAndDownload( activity, lang, null );
    }

}